aboutsummaryrefslogtreecommitdiff
path: root/sway
diff options
context:
space:
mode:
Diffstat (limited to 'sway')
-rw-r--r--sway/commands.c46
-rw-r--r--sway/input/keyboard.c2
-rw-r--r--sway/ipc-server.c3
-rw-r--r--sway/server.c2
4 files changed, 30 insertions, 23 deletions
diff --git a/sway/input/keyboard.c b/sway/input/keyboard.c
index 6dc57d46..99685052 100644
--- a/sway/input/keyboard.c
+++ b/sway/input/keyboard.c
@@ -95,7 +95,7 @@ static void keyboard_execute_command(struct sway_keyboard *keyboard,
binding->command);
config_clear_handler_context(config);
config->handler_context.seat = keyboard->seat_device->sway_seat;
- struct cmd_results *results = handle_command(binding->command);
+ struct cmd_results *results = execute_command(binding->command, NULL);
if (results->status != CMD_SUCCESS) {
wlr_log(L_DEBUG, "could not run command for binding: %s",
binding->command);
diff --git a/sway/ipc-server.c b/sway/ipc-server.c
index ee259c99..4c0953e8 100644
--- a/sway/ipc-server.c
+++ b/sway/ipc-server.c
@@ -336,8 +336,7 @@ void ipc_client_handle_command(struct ipc_client *client) {
case IPC_COMMAND:
{
config_clear_handler_context(config);
- config->handler_context.seat = input_manager_current_seat(input_manager);
- struct cmd_results *results = handle_command(buf);
+ struct cmd_results *results = execute_command(buf, NULL);
const char *json = cmd_results_to_json(results);
char reply[256];
int length = snprintf(reply, sizeof(reply), "%s", json);
diff --git a/sway/server.c b/sway/server.c
index 0753d37e..495769ee 100644
--- a/sway/server.c
+++ b/sway/server.c
@@ -22,7 +22,7 @@ static void server_ready(struct wl_listener *listener, void *data) {
config->active = true;
while (config->cmd_queue->length) {
char *line = config->cmd_queue->items[0];
- struct cmd_results *res = handle_command(line);
+ struct cmd_results *res = execute_command(line, NULL);
if (res->status != CMD_SUCCESS) {
wlr_log(L_ERROR, "Error on line '%s': %s", line, res->error);
}
diff --git a/sway/commands.c b/sway/commands.c
index a7eb6b4a..66614058 100644
--- a/sway/commands.c
+++ b/sway/commands.c
@@ -198,7 +198,7 @@ static struct cmd_handler *find_handler(char *line, enum cmd_status block) {
return res;
}
-struct cmd_results *handle_command(char *_exec) {
+struct cmd_results *execute_command(char *_exec, struct sway_seat *seat) {
// Even though this function will process multiple commands we will only
// return the last error, if any (for now). (Since we have access to an
// error string we could e.g. concatenate all errors there.)
@@ -209,6 +209,16 @@ struct cmd_results *handle_command(char *_exec) {
char *cmd;
list_t *containers = NULL;
+ if (seat == NULL) {
+ // passing a NULL seat means we just pick the default seat
+ seat = sway_input_manager_get_default_seat(input_manager);
+ if (!sway_assert(seat, "could not find a seat to run the command on")) {
+ return NULL;
+ }
+ }
+
+ config->handler_context.seat = seat;
+
head = exec;
do {
// Extract criteria (valid for this command list only).
@@ -278,24 +288,22 @@ struct cmd_results *handle_command(char *_exec) {
if (!has_criteria) {
// without criteria, the command acts upon the focused
// container
- struct sway_seat *seat = config->handler_context.seat;
- if (!seat) {
- seat = sway_input_manager_get_default_seat(input_manager);
+ config->handler_context.current_container =
+ sway_seat_get_focus_inactive(seat, &root_container);
+ if (!sway_assert(config->handler_context.current_container,
+ "could not get focus-inactive for root container")) {
+ return NULL;
}
- if (seat) {
- config->handler_context.current_container =
- sway_seat_get_focus(seat);
- struct cmd_results *res = handler->handle(argc-1, argv+1);
- if (res->status != CMD_SUCCESS) {
- free_argv(argc, argv);
- if (results) {
- free_cmd_results(results);
- }
- results = res;
- goto cleanup;
+ struct cmd_results *res = handler->handle(argc-1, argv+1);
+ if (res->status != CMD_SUCCESS) {
+ free_argv(argc, argv);
+ if (results) {
+ free_cmd_results(results);
}
- free_cmd_results(res);
+ results = res;
+ goto cleanup;
}
+ free_cmd_results(res);
} else {
for (int i = 0; i < containers->length; ++i) {
config->handler_context.current_container = containers->items[i];
@@ -322,13 +330,13 @@ cleanup:
return results;
}
// this is like handle_command above, except:
// 1) it ignores empty commands (empty lines)
// 2) it does variable substitution
// 3) it doesn't split commands (because the multiple commands are supposed to
// be chained together)
// 4) handle_command handles all state internally while config_command has some
// state handled outside (notably the block mode, in read_config)
struct cmd_results *config_command(char *exec, enum cmd_status block) {
struct cmd_results *results = NULL;
int argc;
@@ -322,13 +330,13 @@ cleanup:
return results;
}
-// this is like handle_command above, except:
+// this is like execute_command above, except:
// 1) it ignores empty commands (empty lines)
// 2) it does variable substitution
@@ -322,13 +330,13 @@ cleanup:
// 3) it doesn't split commands (because the multiple commands are supposed to
// be chained together)
-// 4) handle_command handles all state internally while config_command has some
-// state handled outside (notably the block mode, in read_config)
+// 4) execute_command handles all state internally while config_command has
+// some state handled outside (notably the block mode, in read_config)
struct cmd_results *config_command(char *exec, enum cmd_status block) {
struct cmd_results *results = NULL;